#include <ics.h>

/*
 * wrappers for process control functions
 */

pid_t Fork(void) {
  pid_t pid = fork();
  if (pid < 0)
    unix_error("Fork");
  else
    return pid;
}

pid_t Wait(int *status) {
  pid_t pid = wait(status);
  if (pid < 0)
    unix_error("Wait");
  else
    return pid;
}

handler_t *Signal(int signum, handler_t *handler) {
  struct sigaction action, old_action;

  action.sa_handler = handler;  
  sigemptyset(&action.sa_mask); /* only block sig being handled */
  action.sa_flags = SA_RESTART; /* restart syscalls if possible */
  if (sigaction(signum, &action, &old_action) < 0)
    unix_error("Signal");
  else
    return (old_action.sa_handler);
}

void Kill(pid_t pid, int signum) {
  int rc = kill(pid, signum);
  if (rc < 0)
    unix_error("Kill");
}

void Pause() {
  (void)pause();
  return;
}

unsigned int Sleep(unsigned int secs) {
  unsigned int rc = sleep(secs);
  if (rc < 0)
    unix_error("Sleep");
  return rc;
}

/*
 * wrappers for pthreads and Posix semaphore functions
 */
void Pthread_create(pthread_t *tidp, pthread_attr_t *attrp, 
		    void * (*routine)(void *), void *argp) {
  int rc = pthread_create(tidp, attrp, routine, argp);
  if (rc != 0)
    posix_error(rc, "Pthread_create");
}

void Pthread_cancel(pthread_t tid) {
  int rc = pthread_cancel(tid);
  if (rc != 0)
    posix_error(rc, "Pthread_cancel");
}

void Pthread_join(pthread_t tid, void **thread_return) {
  int rc = pthread_join(tid, thread_return);
  if (rc != 0)
    posix_error(rc, "Pthread_join");
}

void Sem_init(sem_t *sem, int pshared, unsigned int value) {
  if (sem_init(sem, pshared, value) < 0)
    unix_error("Sem_init");
}

void P(sem_t *sem) {
  if (sem_wait(sem))
    unix_error("wait full");
}

void V(sem_t *sem) {
  if (sem_post(sem))
    unix_error("post empty");
}

void Pthread_mutex_init(pthread_mutex_t *mutex, 
			pthread_mutexattr_t *attr) {
  int rc = pthread_mutex_init(mutex, attr);
  if (rc != 0)
    posix_error(rc, "Pthread_mutex_init");
}

void Pthread_mutex_lock(pthread_mutex_t *mutex) {
  int rc = pthread_mutex_lock(mutex); 
  if (rc != 0)
    posix_error(rc, "Pthread_mutex_lock");
}

void Pthread_mutex_unlock(pthread_mutex_t *mutex) {
  int rc = pthread_mutex_unlock(mutex); 
  if (rc != 0)
    posix_error(rc, "Pthread_mutex_unlock");
}

void Pthread_cond_init(pthread_cond_t *cond, pthread_condattr_t *attr) {
  int rc = pthread_cond_init(cond, attr);
  if (rc != 0)
    posix_error(rc, "Pthread_cond_init");
}

void Pthread_cond_signal(pthread_cond_t *cond) {
  int rc = pthread_cond_signal(cond);
  if (rc != 0)
    posix_error(rc, "Pthread_cond_signal");
}

void Pthread_cond_broadcast(pthread_cond_t *cond) {
  int rc = pthread_cond_broadcast(cond);
  if (rc != 0)
    posix_error(rc, "Pthread_cond_broadcast");
}

void Pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex) {
  int rc = pthread_cond_wait(cond, mutex);
  if (rc != 0)
    posix_error(rc, "Pthread_cond_wait");
}

int Pthread_cond_timedwait(pthread_cond_t *cond, 
			    pthread_mutex_t *mutex,
			    struct timespec *abstime) {
  int rc = pthread_cond_timedwait(cond, mutex, abstime);
  if ((rc == 0) || (rc == ETIMEDOUT))  
      return rc;
  else 
    posix_error(rc, "Pthread_cond_timedwait");
}  
